import { useLayoutEffect, useEffect, useState, useRef, useContext, useCallback, createContext, useMemo } from 'react';
import { getOverflowAncestors, useFloating as useFloating$1 } from '@floating-ui/react-dom';
export * from '@floating-ui/react-dom';
import { createPortal } from 'react-dom';

var index = typeof document !== 'undefined' ? useLayoutEffect : useEffect;

function reducePropsToArrayWithInitValue(userProps) {
  return Object.entries(userProps != null ? userProps : {}).reduce((acc, _ref) => {
    let [key, value] = _ref;
    acc[key] = [value];
    return acc;
  }, {});
}

function mergeProps(userProps, propsList, elementKey) {
  const mergePropsMap = {
    reference: elementKey === 'reference' ? reducePropsToArrayWithInitValue(userProps) : {},
    floating: elementKey === 'floating' ? reducePropsToArrayWithInitValue(userProps) : {}
  };
  propsList.forEach(props => {
    var _props$elementKey;

    const elementProps = (_props$elementKey = props == null ? void 0 : props[elementKey]) != null ? _props$elementKey : {};
    Object.keys(elementProps).forEach(propKey => {
      if (typeof elementProps[propKey] === 'function') {
        var _mergePropsMap$elemen;

        if (mergePropsMap[elementKey][propKey] == null) {
          mergePropsMap[elementKey][propKey] = [];
        }

        (_mergePropsMap$elemen = mergePropsMap[elementKey][propKey]) == null ? void 0 : _mergePropsMap$elemen.push(elementProps[propKey]);
      }
    });
  });
  return { ...(elementKey === 'floating' && {
      tabIndex: -1
    }),
    ...userProps,
    ...propsList.reduce((acc, props) => {
      Object.assign(acc, props == null ? void 0 : props[elementKey]);
      return acc;
    }, {}),
    ...Object.entries(mergePropsMap[elementKey]).reduce((acc, _ref2) => {
      let [prop, fns] = _ref2;

      if (prop.indexOf('on') === 0) {
        acc[prop] = function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          fns.forEach(fn => fn(...args));
        };
      }

      return acc;
    }, {})
  };
}

const useInteractions = function (propsList) {
  if (propsList === void 0) {
    propsList = [];
  }

  return {
    getReferenceProps: userProps => mergeProps(userProps, propsList, 'reference'),
    getFloatingProps: userProps => mergeProps(userProps, propsList, 'floating')
  };
};

function createTree(nodes) {
  const map = new Map();
  return {
    nodes,

    emit(event, data) {
      var _map$get;

      (_map$get = map.get(event)) == null ? void 0 : _map$get.forEach(handler => handler(data));
    },

    on(event, listener) {
      map.set(event, [...(map.get(event) || []), listener]);
    },

    off(event, listener) {
      map.set(event, (map.get(event) || []).filter(l => l !== listener));
    }

  };
}

var pointInPolygon$1 = {exports: {}};

var flat = function pointInPolygonFlat (point, vs, start, end) {
    var x = point[0], y = point[1];
    var inside = false;
    if (start === undefined) start = 0;
    if (end === undefined) end = vs.length;
    var len = (end-start)/2;
    for (var i = 0, j = len - 1; i < len; j = i++) {
        var xi = vs[start+i*2+0], yi = vs[start+i*2+1];
        var xj = vs[start+j*2+0], yj = vs[start+j*2+1];
        var intersect = ((yi > y) !== (yj > y))
            && (x < (xj - xi) * (y - yi) / (yj - yi) + xi);
        if (intersect) inside = !inside;
    }
    return inside;
};

// ray-casting algorithm based on
// https://wrf.ecse.rpi.edu/Research/Short_Notes/pnpoly.html

var nested = function pointInPolygonNested (point, vs, start, end) {
    var x = point[0], y = point[1];
    var inside = false;
    if (start === undefined) start = 0;
    if (end === undefined) end = vs.length;
    var len = end - start;
    for (var i = 0, j = len - 1; i < len; j = i++) {
        var xi = vs[i+start][0], yi = vs[i+start][1];
        var xj = vs[j+start][0], yj = vs[j+start][1];
        var intersect = ((yi > y) !== (yj > y))
            && (x < (xj - xi) * (y - yi) / (yj - yi) + xi);
        if (intersect) inside = !inside;
    }
    return inside;
};

var pointInPolygonFlat = flat;
var pointInPolygonNested = nested;

pointInPolygon$1.exports = function pointInPolygon (point, vs, start, end) {
    if (vs.length > 0 && Array.isArray(vs[0])) {
        return pointInPolygonNested(point, vs, start, end);
    } else {
        return pointInPolygonFlat(point, vs, start, end);
    }
};
pointInPolygon$1.exports.nested = pointInPolygonNested;
pointInPolygon$1.exports.flat = pointInPolygonFlat;

var pointInPolygon = pointInPolygon$1.exports;

function getDocument(floating) {
  var _floating$ownerDocume;

  return (_floating$ownerDocume = floating == null ? void 0 : floating.ownerDocument) != null ? _floating$ownerDocume : document;
}

function getWindow(value) {
  var _getDocument$defaultV;

  return (_getDocument$defaultV = getDocument(value).defaultView) != null ? _getDocument$defaultV : window;
}

function isElement(value) {
  return value ? value instanceof getWindow(value).Element : false;
}
function isHTMLElement(value) {
  return value ? value instanceof getWindow(value).HTMLElement : false;
}

function safePolygon(_temp) {
  let {
    timeout = 0,
    debug = null
  } = _temp === void 0 ? {} : _temp;
  let timeoutId;
  return _ref => {
    let {
      x,
      y,
      placement,
      refs,
      onClose,
      treeRef,
      nodeRef
    } = _ref;
    return function onPointerMove(event) {
      var _refs$floating$curren;

      if (event.pointerType === 'touch') {
        return;
      }

      const {
        target,
        clientX,
        clientY
      } = event;
      const targetNode = target;

      if (event.type === 'pointermove' && isElement(refs.reference.current) && refs.reference.current.contains(targetNode) || (_refs$floating$curren = refs.floating.current) != null && _refs$floating$curren.contains(targetNode)) {
        return;
      } // If any child has a menu open, abort


      if (treeRef != null && treeRef.current.nodes.filter(_ref2 => {
        let {
          current
        } = _ref2;
        return current.parentId === (nodeRef == null ? void 0 : nodeRef.current.id);
      }).some(_ref3 => {
        let {
          current
        } = _ref3;
        return current.open;
      })) {
        return;
      }

      if (!refs.reference.current || !refs.floating.current || placement == null || x == null || y == null) {
        return;
      }

      const refRect = refs.reference.current.getBoundingClientRect();
      const rect = refs.floating.current.getBoundingClientRect();
      const side = placement.split('-')[0];
      const cursorLeaveFromRight = x > rect.right - rect.width / 2;
      const cursorLeaveFromBottom = y > rect.bottom - rect.height / 2;
      const BUFFER = 1; // Within the rectangular trough between the two elements

      switch (side) {
        case 'top':
          if (clientX >= rect.left && clientX <= rect.right && clientY >= rect.bottom && clientY <= refRect.top) {
            return;
          }

          break;

        case 'bottom':
          if (clientX >= rect.left && clientX <= rect.right && clientY >= refRect.bottom && clientY <= rect.top) {
            return;
          }

          break;

        case 'left':
          if (clientX >= rect.right && clientX <= refRect.left && clientY >= rect.left && clientY <= rect.right) {
            return;
          }

          break;

        case 'right':
          if (clientX >= rect.right && clientX <= refRect.left && clientY >= rect.right && clientY <= refRect.left) {
            return;
          }

          break;
      }

      function getPolygon(_ref4) {
        let [x, y] = _ref4;

        switch (side) {
          case 'top':
            {
              const cursorPointOne = [x - (cursorLeaveFromRight ? -BUFFER : BUFFER), y + BUFFER];
              const cursorPointTwo = [x - (cursorLeaveFromRight ? -BUFFER : BUFFER), y - BUFFER];
              const commonPoints = [[rect.left, cursorLeaveFromRight ? rect.bottom : rect.width > refRect.width ? rect.bottom : rect.top], [rect.right, cursorLeaveFromRight ? rect.width > refRect.width ? rect.bottom : rect.top : rect.bottom]];

              if (rect.width < refRect.width) {
                return cursorLeaveFromRight ? [cursorPointOne, ...commonPoints, cursorPointTwo] : [cursorPointOne, cursorPointTwo, ...commonPoints];
              }

              return cursorLeaveFromRight ? [cursorPointOne, ...commonPoints] : [cursorPointOne, ...commonPoints];
            }

          case 'bottom':
            {
              const cursorPointOne = [x - (cursorLeaveFromRight ? -BUFFER : BUFFER), y - BUFFER];
              const cursorPointTwo = [x - (cursorLeaveFromRight ? -BUFFER : BUFFER), y + BUFFER];
              const commonPoints = [[rect.left, cursorLeaveFromRight ? rect.top : rect.width > refRect.width ? rect.top : rect.bottom], [rect.right, cursorLeaveFromRight ? rect.width > refRect.width ? rect.top : rect.bottom : rect.top]];

              if (rect.width < refRect.width) {
                return cursorLeaveFromRight ? [cursorPointOne, ...commonPoints, cursorPointTwo] : [cursorPointOne, cursorPointTwo, ...commonPoints];
              }

              return cursorLeaveFromRight ? [cursorPointOne, ...commonPoints] : [cursorPointOne, ...commonPoints];
            }

          case 'left':
            {
              const cursorPointOne = [x + BUFFER, y - (cursorLeaveFromBottom ? -BUFFER : BUFFER)];
              const cursorPointTwo = [x - BUFFER, y - (cursorLeaveFromBottom ? -BUFFER : BUFFER)];
              const commonPoints = [[cursorLeaveFromBottom ? rect.right - BUFFER : rect.height > refRect.height ? rect.right - BUFFER : rect.left, rect.top], [cursorLeaveFromBottom ? rect.height > refRect.height ? rect.right - BUFFER : rect.left : rect.right - BUFFER, rect.bottom]];

              if (rect.height < refRect.height) {
                return cursorLeaveFromBottom ? [cursorPointOne, ...commonPoints, cursorPointTwo] : [cursorPointOne, cursorPointTwo, ...commonPoints];
              }

              return cursorLeaveFromBottom ? [cursorPointOne, ...commonPoints] : [cursorPointOne, ...commonPoints];
            }

          case 'right':
            {
              const cursorPointOne = [x - BUFFER, y - (cursorLeaveFromBottom ? -BUFFER : BUFFER)];
              const cursorPointTwo = [x + BUFFER, y - (cursorLeaveFromBottom ? -BUFFER : BUFFER)];
              const commonPoints = [[cursorLeaveFromBottom ? rect.left + BUFFER : rect.height > refRect.height ? rect.left + BUFFER : rect.right, rect.top], [cursorLeaveFromBottom ? rect.height > refRect.height ? rect.left + BUFFER : rect.right : rect.left + BUFFER, rect.bottom]];

              if (rect.height < refRect.height) {
                return cursorLeaveFromBottom ? [cursorPointOne, ...commonPoints, cursorPointTwo] : [cursorPointOne, cursorPointTwo, ...commonPoints];
              }

              return cursorLeaveFromBottom ? [cursorPointOne, ...commonPoints] : [cursorPointOne, ...commonPoints];
            }
        }
      }

      const poly = getPolygon([x, y]);

      if (process.env.NODE_ENV !== "production") {
        debug == null ? void 0 : debug(getPolygon([x, y]).slice(0, 4).join(', '));
      }

      if (!pointInPolygon([clientX, clientY], poly)) {
        clearTimeout(timeoutId);
        onClose();
      } else if (timeout) {
        timeoutId = setTimeout(onClose, timeout);
      }
    };
  };
}

function FloatingPortal(_ref) {
  let {
    children
  } = _ref;
  const [mounted, setMounted] = useState(false);
  const portalRef = useRef(null);
  index(() => {
    const root = document.getElementById('floating-ui-root');

    if (root) {
      portalRef.current = root;
    } else {
      portalRef.current = document.createElement('div');
      portalRef.current.id = 'floating-ui-root';
    }

    const el = portalRef.current;

    if (!document.body.contains(el)) {
      document.body.appendChild(el);
    }

    setMounted(true);
    return () => {
      if (document.body.contains(el)) {
        document.body.removeChild(el);
      }
    };
  }, []);

  if (mounted && portalRef.current) {
    return /*#__PURE__*/createPortal(children, portalRef.current);
  }

  return null;
}

function getDelay(value, prop) {
  if (typeof value === 'number') {
    return value;
  }

  return value[prop];
}
const useHover = function (context, _temp) {
  let {
    delay = 0,
    handleLeave = null,
    mouseOnly = false,
    pointerRestMs = 0
  } = _temp === void 0 ? {} : _temp;
  const {
    onOpenChange = () => {},
    open,
    dataRef,
    refs
  } = context;
  const enterEventRef = useRef(null);
  const hoveredThenClickedRef = useRef(false);
  const timeoutRef = useRef();
  const handlerRef = useRef();
  index(() => {
    if (!handleLeave) {
      return;
    }

    if (!open) {
      hoveredThenClickedRef.current = false;
      return;
    } // If the cursor is moving quickly off the page, the `pointermove` event
    // to check if the cursor is outside of the "safe triangle" area will not
    // fire, resulting in the floating element remaining open.


    function onHTMLPointerLeave() {
      var _enterEventRef$curren;

      if (((_enterEventRef$curren = enterEventRef.current) == null ? void 0 : _enterEventRef$curren.type) === 'pointerover') {
        onOpenChange(false);
      }
    }

    const html = getDocument(refs.floating.current).documentElement;
    html.addEventListener('pointerleave', onHTMLPointerLeave);
    return () => {
      html.removeEventListener('pointerleave', onHTMLPointerLeave);
    };
  }, [open, onOpenChange, handleLeave]);
  const pointerRestTimeoutRef = useRef();
  return {
    reference: {
      onPointerMove() {
        if (pointerRestMs === 0 || open) {
          return;
        }

        clearTimeout(pointerRestTimeoutRef.current);
        pointerRestTimeoutRef.current = setTimeout(() => {
          if (!dataRef.current.dismiss) {
            onOpenChange(true);
          }
        }, pointerRestMs);
      },

      onPointerEnter(event) {
        if (mouseOnly && event.pointerType !== 'mouse') {
          return;
        }

        enterEventRef.current = event.nativeEvent;

        if (delay) {
          clearTimeout(timeoutRef.current);
          timeoutRef.current = setTimeout(() => {
            if (!dataRef.current.dismiss) {
              onOpenChange(true);
            }
          }, getDelay(delay, 'open'));
        } else {
          onOpenChange(true);
        }
      },

      onPointerLeave(event) {
        var _enterEventRef$curren2;

        if (hoveredThenClickedRef.current || ((_enterEventRef$curren2 = enterEventRef.current) == null ? void 0 : _enterEventRef$curren2.type) === 'click') {
          return;
        }

        const doc = getDocument(refs.floating.current);

        if (handleLeave) {
          clearTimeout(timeoutRef.current);
          handlerRef.current && doc.removeEventListener('pointermove', handlerRef.current);
          handlerRef.current = handleLeave({ ...context,
            x: event.clientX,
            y: event.clientY,

            onClose() {
              handlerRef.current && doc.removeEventListener('pointermove', handlerRef.current);
              onOpenChange(false);
            }

          });
          doc.addEventListener('pointermove', handlerRef.current);
          return;
        }

        dataRef.current.dismiss = false;

        if (delay) {
          clearTimeout(pointerRestTimeoutRef.current);
          clearTimeout(timeoutRef.current);
          timeoutRef.current = setTimeout(() => onOpenChange(false), getDelay(delay, 'close'));
        } else {
          hoveredThenClickedRef.current = false;
          onOpenChange(false);
        }
      }

    }
  };
};

const FloatingGroupContext = /*#__PURE__*/createContext({
  delay: 1000,
  initialDelay: 1000,
  currentId: null,
  setCurrentId: () => {},
  setState: () => {}
});
const useGroupContext = () => {
  const context = useContext(FloatingGroupContext);

  if (!context) {
    throw new Error('useGroupContext() must be used within a <FloatingGroup />');
  }

  return context;
};
const FloatingGroup = _ref => {
  let {
    children,
    delay
  } = _ref;
  const [state, setState] = useState({
    delay,
    initialDelay: delay,
    currentId: null
  });
  const setCurrentId = useCallback(currentId => {
    setState(state => ({ ...state,
      currentId
    }));
  }, []);
  return /*#__PURE__*/React.createElement(FloatingGroupContext.Provider, {
    value: { ...state,
      setState,
      setCurrentId
    }
  }, children);
};
const useGroup = (_ref2, _ref3) => {
  let {
    open,
    onOpenChange
  } = _ref2;
  let {
    id
  } = _ref3;
  const {
    currentId,
    initialDelay,
    setState
  } = useGroupContext();
  useEffect(() => {
    if (currentId && onOpenChange) {
      setState(state => ({ ...state,
        delay: {
          open: 0,
          close: getDelay(initialDelay, 'close')
        }
      }));

      if (currentId !== id) {
        onOpenChange(false);
      }
    }
  }, [id, onOpenChange, setState, currentId, initialDelay]);
  useEffect(() => {
    if (!open && currentId === id && onOpenChange) {
      onOpenChange(false);
      setState(state => ({ ...state,
        delay: initialDelay,
        currentId: null
      }));
    }
  }, [open, setState, currentId, id, onOpenChange, initialDelay]);
};

let globalId = 0;

function genId() {
  return "floating-ui-" + globalId++;
}

function useFloatingId() {
  return useMemo(genId, []);
}

const useAria = function (_ref, _temp) {
  let {
    open
  } = _ref;
  let {
    role = 'dialog',
    titleId,
    descriptionId
  } = _temp === void 0 ? {} : _temp;
  const rootId = useFloatingId();
  const floatingProps = {
    id: rootId,
    role
  };

  if (role === 'tooltip') {
    return {
      reference: {
        'aria-describedby': open ? rootId : undefined
      },
      floating: floatingProps
    };
  }

  return {
    reference: {
      'aria-expanded': open ? 'true' : 'false',
      'aria-haspopup': role,
      'aria-controls': open ? rootId : undefined,
      ...(role === 'listbox' && {
        role: 'combobox'
      })
    },
    floating: { ...floatingProps,
      ...(role === 'dialog' && {
        'aria-labelledby': titleId,
        'aria-describedby': descriptionId
      })
    }
  };
};

const useClick = _ref => {
  let {
    open,
    onOpenChange = () => {},
    dataRef
  } = _ref;
  return {
    reference: {
      onClick(event) {
        if (open) {
          var _dataRef$current$open;

          if (((_dataRef$current$open = dataRef.current.openEvent) == null ? void 0 : _dataRef$current$open.type) === 'click' || dataRef.current.hoveredThenClicked) {
            dataRef.current.hoveredThenClicked = false;
            onOpenChange(false);
          } else {
            // Clicking to keep it open after a hover.
            dataRef.current.hoveredThenClicked = true;
          }
        } else {
          dataRef.current.openEvent = event.nativeEvent;
          onOpenChange(true);
        }
      }

    }
  };
};

const useDismiss = function (_ref, _temp) {
  let {
    onOpenChange = () => {},
    refs,
    open,
    treeRef,
    nodeRef,
    dataRef
  } = _ref;
  let {
    escapeKey = true,
    outsidePointerDown = true,
    referencePointerDown = false,
    ancestorScroll = false
  } = _temp === void 0 ? {} : _temp;
  const focusReference = useCallback(() => {
    if (isHTMLElement(refs.reference.current)) {
      refs.reference.current.focus();
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [refs.reference.current]);
  useEffect(() => {
    function onClose() {
      onOpenChange(false);
      focusReference();
    }

    const tree = treeRef == null ? void 0 : treeRef.current;
    tree == null ? void 0 : tree.on('close', onClose);
    return () => {
      tree == null ? void 0 : tree.off('close', onClose);
    };
  }, [treeRef, onOpenChange, focusReference]);
  useEffect(() => {
    if (!open) {
      return;
    }

    function isAnyChildNodeOpen() {
      return treeRef == null ? void 0 : treeRef.current.nodes.filter(_ref2 => {
        let {
          current
        } = _ref2;
        return current.parentId === (nodeRef == null ? void 0 : nodeRef.current.id);
      }).some(_ref3 => {
        let {
          current
        } = _ref3;
        return current.open;
      });
    }

    function onKeyDown(event) {
      if (event.key === 'Escape' && !isAnyChildNodeOpen()) {
        dataRef.current.dismiss = true;

        if (treeRef) {
          treeRef.current.emit('close');
        } else {
          onOpenChange(false);
          focusReference();
        }
      }
    }

    function onPointerDown(event) {
      var _refs$floating$curren;

      if ((_refs$floating$curren = refs.floating.current) != null && _refs$floating$curren.contains(event.target) || isElement(refs.reference.current) && refs.reference.current.contains(event.target) || isAnyChildNodeOpen()) {
        return;
      }

      if (treeRef) {
        treeRef.current.emit('close');
      } else {
        onOpenChange(false);
        focusReference();
      }
    }

    function onScroll() {
      onOpenChange(false);
    }

    const doc = getDocument(refs.floating.current);
    escapeKey && doc.addEventListener('keydown', onKeyDown);
    outsidePointerDown && doc.addEventListener('pointerdown', onPointerDown);
    const ancestors = ancestorScroll ? [...(isElement(refs.reference.current) ? getOverflowAncestors(refs.reference.current) : []), ...(isElement(refs.floating.current) ? getOverflowAncestors(refs.floating.current) : [])] : [];
    ancestors.forEach(ancestor => ancestor.addEventListener('scroll', onScroll, {
      passive: true
    }));
    return () => {
      escapeKey && doc.removeEventListener('keydown', onKeyDown);
      outsidePointerDown && doc.removeEventListener('pointerdown', onPointerDown);
      ancestors.forEach(ancestor => ancestor.removeEventListener('scroll', onScroll));
    };
  }, [escapeKey, outsidePointerDown, treeRef, nodeRef, open, onOpenChange, dataRef, focusReference, ancestorScroll, refs.floating, refs.reference]);
  return {
    reference: {
      onPointerDown() {
        if (referencePointerDown) {
          dataRef.current.dismiss = true;
          onOpenChange(false);
        }
      }

    }
  };
};

const useFocus = function (_ref, _temp) {
  let {
    open,
    onOpenChange = () => {},
    dataRef,
    refs
  } = _ref;
  let {
    keyboardOnly = true
  } = _temp === void 0 ? {} : _temp;
  const pointerTypeRef = useRef();
  index(() => {
    if (!open) {
      pointerTypeRef.current = undefined;
    }
  });
  return {
    reference: {
      onPointerDown(_ref2) {
        let {
          pointerType
        } = _ref2;
        // undefined or "" depending on the browser
        pointerTypeRef.current = pointerType;
      },

      onFocus(event) {
        var _refs$reference$curre, _dataRef$current$open;

        if (keyboardOnly && pointerTypeRef.current) {
          return;
        } // Dismiss with click should ignore the subsequent `focus` trigger, but
        // only if the click originated inside the reference element.


        if (event.type === 'focus' && dataRef.current.openEvent === 'mousedown' && isElement(refs.reference.current) && (_refs$reference$curre = refs.reference.current) != null && _refs$reference$curre.contains((_dataRef$current$open = dataRef.current.openEvent) == null ? void 0 : _dataRef$current$open.target)) {
          return;
        }

        if (!dataRef.current.dismiss) {
          dataRef.current.openEvent = event.nativeEvent;
          onOpenChange(true);
        }
      },

      onBlur(event) {
        var _refs$floating$curren;

        const target = event.relatedTarget; // When focusing the reference element (e.g. regular click), then
        // clicking into the floating element, prevent it from hiding.
        // Note: it must be focusable, e.g. `tabindex="-1"`.

        if ((_refs$floating$curren = refs.floating.current) != null && _refs$floating$curren.contains(target) || isElement(refs.reference.current) && refs.reference.current.contains(target)) {
          return;
        }

        dataRef.current.dismiss = false;
        dataRef.current.hoveredThenClicked = false;
        onOpenChange(false);
      }

    }
  };
};

const FOCUSABLE_ELEMENT_SELECTOR = 'a[href],area[href],input:not([disabled]),select:not([disabled]),textarea:not([disabled]),button:not([disabled]),iframe,object,embed,*[tabindex],*[contenteditable]';
const useFocusTrap = function (_ref, _temp) {
  let {
    open,
    refs,
    treeRef,
    nodeRef,
    onOpenChange
  } = _ref;
  let {
    initialContentFocus = 0,
    order = ['content'],
    modal = true,
    inert = false,
    scrollLock = 'auto'
  } = _temp === void 0 ? {} : _temp;
  const indexRef = useRef(0);
  const getFocusableElements = useCallback(() => {
    return order.map(type => {
      if (isHTMLElement(refs.reference.current) && type === 'reference') {
        return refs.reference.current;
      }

      if (refs.floating.current && type === 'floating') {
        return refs.floating.current;
      }

      if (type === 'content') {
        var _refs$floating$curren, _refs$floating$curren2;

        return Array.from((_refs$floating$curren = (_refs$floating$curren2 = refs.floating.current) == null ? void 0 : _refs$floating$curren2.querySelectorAll(FOCUSABLE_ELEMENT_SELECTOR)) != null ? _refs$floating$curren : []);
      }

      return null;
    }).filter(Boolean).flat();
  }, [refs.floating, refs.reference, order]);
  useEffect(() => {
    if (inert) {
      return;
    }

    if (open) {
      const focusableElements = getFocusableElements();

      if (typeof initialContentFocus === 'number') {
        var _focusableElements$in;

        (_focusableElements$in = focusableElements[initialContentFocus]) == null ? void 0 : _focusableElements$in.focus({
          preventScroll: true
        });
      } else if (initialContentFocus.current) {
        var _focusableElements$fi;

        (_focusableElements$fi = focusableElements.find(element => element === initialContentFocus.current)) == null ? void 0 : _focusableElements$fi.focus({
          preventScroll: true
        });
      }
    } else if (modal && isHTMLElement(refs.reference.current)) {
      refs.reference.current.focus();
    }
  }, [getFocusableElements, open, inert, modal, initialContentFocus, refs.reference]);
  useEffect(() => {
    if (!modal) {
      return;
    }

    if (!open) {
      indexRef.current = 0;
      return;
    }

    function onKeyDown(event) {
      if (treeRef != null && treeRef.current.nodes.filter(_ref2 => {
        let {
          current
        } = _ref2;
        return current.parentId === (nodeRef == null ? void 0 : nodeRef.current.id);
      }).some(_ref3 => {
        let {
          current
        } = _ref3;
        return current.open;
      })) {
        return;
      }

      if (event.key === 'Tab') {
        var _focusableElements$in2;

        event.preventDefault();
        event.stopPropagation();

        if (inert) {
          return;
        }

        const focusableElements = getFocusableElements();

        if (event.shiftKey) {
          indexRef.current = indexRef.current === 0 ? focusableElements.length - 1 : indexRef.current - 1;
        } else {
          indexRef.current = indexRef.current === focusableElements.length - 1 ? 0 : indexRef.current + 1;
        }

        (_focusableElements$in2 = focusableElements[indexRef.current]) == null ? void 0 : _focusableElements$in2.focus({
          preventScroll: true
        });
      }
    }

    const doc = getDocument(refs.floating.current);
    doc.addEventListener('keydown', onKeyDown);
    return () => {
      doc.removeEventListener('keydown', onKeyDown);
    };
  }, [getFocusableElements, nodeRef, open, modal, treeRef, inert, refs.floating]);

  function onBlur(event) {
    var _refs$floating$curren3;

    const target = event.relatedTarget;

    if (target && !((_refs$floating$curren3 = refs.floating.current) != null && _refs$floating$curren3.contains(target)) && isElement(refs.reference.current) && !refs.reference.current.contains(target)) {
      onOpenChange == null ? void 0 : onOpenChange(false);
    }
  }

  useEffect(() => {
    if (!open || !modal) {
      return;
    }

    const doc = getDocument(refs.floating.current);
    const nodes = doc.querySelectorAll('body > *:not(#floating-ui-root)');
    nodes.forEach(node => {
      node.setAttribute('aria-hidden', 'true');
    });
    return () => {
      nodes.forEach(node => {
        node.removeAttribute('aria-hidden');
      });
    };
  }, [open, modal, scrollLock, refs.floating]);
  index(() => {
    if (!open) {
      return;
    }

    const applyScrollLock = scrollLock === true || scrollLock === 'auto' && modal;
    const doc = getDocument(refs.floating.current);

    if (applyScrollLock) {
      var _doc$defaultView;

      const scrollbarWidth = ((_doc$defaultView = doc.defaultView) != null ? _doc$defaultView : window).innerWidth - doc.documentElement.offsetWidth;
      doc.documentElement.style.overflow = 'hidden';
      doc.body.style.paddingRight = scrollbarWidth + "px";
    }

    return () => {
      if (applyScrollLock) {
        doc.documentElement.style.overflow = '';
        doc.body.style.paddingRight = '';
      }
    };
  }, [open, scrollLock]);

  if (modal) {
    return {
      floating: {
        'aria-modal': 'true'
      }
    };
  }

  return {
    reference: {
      onBlur
    },
    floating: {
      onBlur
    }
  };
};

function stop(event) {
  event.preventDefault();
  event.stopPropagation();
}

function findNonDisabledIndex(listRef, _temp) {
  let {
    startingIndex = -1,
    decrement = false
  } = _temp === void 0 ? {} : _temp;
  let index = startingIndex;

  do {
    var _listRef$current$inde, _listRef$current$inde2;

    index = index + (decrement ? -1 : 1);
  } while ((_listRef$current$inde = listRef.current[index]) != null && _listRef$current$inde.hasAttribute('disabled') || ((_listRef$current$inde2 = listRef.current[index]) == null ? void 0 : _listRef$current$inde2.getAttribute('aria-disabled')) === 'true');

  return index === -1 ? 0 : index;
}

const useListNavigation = (_ref, _ref2) => {
  let {
    open,
    onOpenChange = () => {},
    refs
  } = _ref;
  let {
    listRef,
    activeIndex = null,
    selectedIndex = null,
    loop = false,
    nested = false,
    rtl = false,
    preventScroll = true,
    onNavigate = () => {}
  } = _ref2;
  const focusOnOpenRef = useRef(true);
  const indexRef = useRef(selectedIndex != null ? selectedIndex : 0);
  const keyRef = useRef('');
  const focusItem = useCallback((listRef, indexRef) => {
    var _listRef$current$inde3;

    (_listRef$current$inde3 = listRef.current[indexRef.current]) == null ? void 0 : _listRef$current$inde3.focus({
      preventScroll
    });
  }, [preventScroll]);
  index(() => {
    if (selectedIndex != null) {
      indexRef.current = selectedIndex;
    }

    if (open && focusOnOpenRef.current) {
      onNavigate(indexRef.current);
      focusItem(listRef, indexRef);
    }
  }, [open, selectedIndex, listRef, onNavigate, focusItem]);
  index(() => {
    if (open && activeIndex != null) {
      indexRef.current = activeIndex;
      onNavigate(indexRef.current);
      focusItem(listRef, indexRef);
    }
  }, [open, activeIndex, listRef, onNavigate, focusItem]);
  index(() => {
    if (selectedIndex != null) {
      return;
    }

    if (open) {
      if (keyRef.current === 'ArrowDown' || keyRef.current === 'ArrowUp' || focusOnOpenRef.current && (keyRef.current === ' ' || keyRef.current === 'Enter')) {
        indexRef.current = keyRef.current === 'ArrowUp' ? findNonDisabledIndex(listRef, {
          startingIndex: listRef.current.length,
          decrement: true
        }) : findNonDisabledIndex(listRef);
        onNavigate(indexRef.current);
        focusItem(listRef, indexRef);
      }
    }

    keyRef.current = '';
  }, [open, listRef, selectedIndex, onNavigate, focusItem]);
  index(() => {
    if (!open && selectedIndex != null) {
      var _refs$reference$curre;

      (_refs$reference$curre = refs.reference.current) == null ? void 0 : _refs$reference$curre.focus({
        preventScroll
      });
    }
  }, [refs.reference, preventScroll, selectedIndex, open]);
  index(() => {
    if (!open) {
      focusOnOpenRef.current = true;
    }
  }, [open]);
  return {
    reference: {
      onPointerDown(event) {
        // undefined or '' depending on the browser
        focusOnOpenRef.current = !event.pointerType;
      },

      onKeyDown(event) {
        keyRef.current = event.key;

        if (refs.reference.current instanceof Element && refs.reference.current.tagName !== 'BUTTON') {
          if (event.key === 'Enter' || event.key === ' ') {
            onOpenChange(true);
          }
        }

        if (nested) {
          if (event.key === (rtl ? 'ArrowLeft' : 'ArrowRight')) {
            indexRef.current = findNonDisabledIndex(listRef);
            stop(event);
            onOpenChange(true);
            onNavigate(indexRef.current);
            focusItem(listRef, indexRef);
          }

          return;
        }

        if (event.key === 'ArrowDown' || event.key === 'ArrowUp') {
          if (selectedIndex == null) {
            indexRef.current = event.key === 'ArrowDown' ? findNonDisabledIndex(listRef) : findNonDisabledIndex(listRef, {
              startingIndex: listRef.current.length,
              decrement: true
            });
          } else {
            indexRef.current = selectedIndex;
          }

          stop(event);
          onOpenChange(true);
          onNavigate(indexRef.current);
          focusItem(listRef, indexRef);
        }
      }

    },
    floating: {
      onKeyDown(event) {
        if (nested && event.key === (rtl ? 'ArrowRight' : 'ArrowLeft')) {
          stop(event);
          onOpenChange(false);

          if (isHTMLElement(refs.reference.current)) {
            refs.reference.current.focus();
          }

          return;
        }

        const currentIndex = indexRef.current;
        const minIndex = findNonDisabledIndex(listRef);
        const maxIndex = findNonDisabledIndex(listRef, {
          decrement: true,
          startingIndex: listRef.current.length
        });

        if (event.key === 'Home') {
          indexRef.current = minIndex;
          onNavigate(indexRef.current);
          focusItem(listRef, indexRef);
        }

        if (event.key === 'End') {
          indexRef.current = maxIndex;
          onNavigate(indexRef.current);
          focusItem(listRef, indexRef);
        }

        if (event.key === 'ArrowDown' || event.key === 'ArrowUp') {
          stop(event);

          if (event.currentTarget.ownerDocument.activeElement === event.currentTarget && selectedIndex != null) {
            indexRef.current = selectedIndex;
            onNavigate(indexRef.current);
            focusItem(listRef, indexRef);
            return;
          }

          if (event.key === 'ArrowDown') {
            if (loop) {
              indexRef.current = currentIndex === maxIndex ? minIndex : findNonDisabledIndex(listRef, {
                startingIndex: currentIndex
              });
            } else {
              indexRef.current = Math.min(maxIndex, findNonDisabledIndex(listRef, {
                startingIndex: currentIndex
              }));
            }
          } else {
            if (loop) {
              indexRef.current = currentIndex === minIndex ? maxIndex : findNonDisabledIndex(listRef, {
                startingIndex: currentIndex,
                decrement: true
              });
            } else {
              indexRef.current = Math.max(minIndex, findNonDisabledIndex(listRef, {
                startingIndex: currentIndex,
                decrement: true
              }));
            }
          }

          onNavigate(indexRef.current);
          focusItem(listRef, indexRef);
        }
      }

    }
  };
};

const useTypeahead = (context, _ref) => {
  let {
    listRef,
    onMatch,
    debounceMs = 500,
    ignoreKeys = []
  } = _ref;
  const timeoutIdRef = useRef();
  const stringRef = useRef('');
  const prevIndexRef = useRef(null);
  const matchIndexRef = useRef(null);

  function onKeyDown(event) {
    if (event.key === ' ' && stringRef.current.length > 0) {
      event.preventDefault();
      event.stopPropagation();
    }

    const listContent = listRef.current;

    if (listContent == null || ['Home', 'End', 'Escape', 'Enter', 'Tab', 'ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight', ...ignoreKeys].includes(event.key)) {
      return;
    }

    stringRef.current += event.key;
    clearTimeout(timeoutIdRef.current);
    timeoutIdRef.current = setTimeout(() => {
      stringRef.current = '';
      prevIndexRef.current = matchIndexRef.current;
    }, debounceMs);
    const prevIndex = prevIndexRef.current;
    const str = [...listContent.slice((prevIndex != null ? prevIndex : -1) + 1), ...listContent.slice(0, prevIndex != null ? prevIndex : 0)].find(txt => (txt == null ? void 0 : txt.toLowerCase().indexOf(stringRef.current)) === 0);
    const index = str ? listContent.indexOf(str) : -1;

    if (index !== -1) {
      onMatch(index);
      matchIndexRef.current = index;
    }
  }

  return {
    reference: {
      onKeyDown
    },
    floating: {
      onKeyDown
    }
  };
};

function useFloating(_temp) {
  let {
    open = false,
    onOpenChange,
    placement,
    middleware,
    strategy,
    treeRef,
    nodeRef
  } = _temp === void 0 ? {} : _temp;
  index(() => {
    if (nodeRef) {
      nodeRef.current.open = open;
    }
  });
  const dataRef = useRef({});
  const floating = useFloating$1({
    placement,
    middleware,
    strategy
  });
  const context = useMemo(() => ({ ...floating,
    treeRef,
    nodeRef,
    open,
    onOpenChange,
    dataRef
  }), [floating, nodeRef, treeRef, open, onOpenChange]);
  return useMemo(() => ({
    context,
    ...floating
  }), [floating, context]);
}

export { FloatingGroup, FloatingPortal, createTree, safePolygon, useAria, useClick, useDismiss, useFloating, useFloatingId, useFocus, useFocusTrap, useGroup, useGroupContext, useHover, useInteractions, useListNavigation, useTypeahead };
